--[[
	Author:  Azmaedus
	Version: 0.4.2b
	compatible with game version 6.0.3
	
	Change Log
	----------
	11/29/2014 - 0.1 	Creation of addon
	11/29/2014 - 0.1.1 	Support for tier 1, 2, and 3 garrison (Outpost, Fort, and Garrison)
	12/01/2014 - r8		1st commit to wowace.com (alpha)
	12/02/2014 - r9		Added multi-dimensional table to hold music filename references
	12/26/2014 - r9		Fixed issue with music not playing inside garrison buildings by changing the way to detect if we are inside Lunarfall
	12/27/2014 - r10	2 minute timer added between random playlist items.
						Dropdown music now functional per expansion selected.
						Fixed reload (musiclist) not loading in profile; forgot DB at the end of savedvars .toc
	12/28/2014 - r11	Fixed /reload issue not playing music
	12/29/2014 - 0.2b	1st beta: Fixed error when opening the options UI
	12/29/2014 - 0.3b	Changed from using 2 table design to a single table for holding music information
	12/29/2014 - 0.3.1b	Added Evil Forest 1, 2 and 3 music (Silverpine & Tirisfal)
	12/30/2014 - 0.4b	Code review & optimization (1st pass)
						Re-design of options UI
						Added Audio tab to let users play a wow embedded file by copy/paste into the input control
						Added About tab with relative information
						Localization of music titles and options table.
	12/31/2014 - 0.4.1b	Fixed issue with options UI not showing up on first /gjb command
						Added playlist controls in audio tab
						Added more music to the musictable
						Added slash commands play, prev, next, stop
						Fixed small misc bugs and some naughty coding practices
						Started to implement a zonelist so the user can choose to play the jukebox in multiple locations.
	12/31/2014 - 0.4.2b	Removed unecessary xml frame file to trigger OnLoad event
						Moved audio player in the jukebox tab
						Added a function with enums to manage string colors
						Added more music
						
						
	Known Bugs
	----------
]]

-- Load the Ace libraries --
GarrisonJukeBox = LibStub("AceAddon-3.0"):NewAddon("GarrisonJukeBox", "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("GarrisonJukeBox")

-- globals
local gVersion = "0.4.2b"
local gReleaseDate = "12/31/2014"
local gMusicPlaying = false 	-- is the music currently playing
local gMusicIndex = 1			-- current item index of table musicfiles
local gMaxMusic = 0				-- number of items in musicfiles table
local gOnLoadMusicTimer = 5		-- HACK!!! wait n seconds when PLAYYER_LOGIN is triggered before playing music (make sure Blizz level loading script doesn't override PlayMusic() call)
local LUNARFALL = 971			-- zone id of Lunarfall

-- function to set the color of text provided
COLORS = {
	SG_GREEN = "|cff38ce39",
	SG_BLUE = "|cff539ce8",
	SG_WHITE = "|cffffffff",
	SG_BLACK = "|cff000000",
}
function setTextColor(s,c)
	if c ~= nil or c ~= "" then
		-- make sure the color exist, if not, set to white
		if COLORS[c] == nil then
			assert(COLORS[c], c .. L["ERROR_INVALID_COLOR"])
			c = COLORS["SG_WHITE"]
		end
		
		s = COLORS[c] .. s .. "|r"
	end
	
	return s
end

-- temp storage for select vars
local expac = "i1wow"
local music = 1
local jukebox = ""
local playsoundfile = ""
local doubleslashes = true
local zone = "i1zone"
local zonebox = ""

-- Create the default settings for this addon --
local defaults = { 
    profile =  {
		musiclist = {},
		--zonelist = {},
		shuffle = false,
	},
}

-- Get localization
local L = LibStub("AceLocale-3.0"):GetLocale("GarrisonJukeBox")

-- table holding music info (name, file, length)
local musictable = {
	-- Classic
	i1wow = {
		{L["TITLE_MAIN_THEME"], "Sound\\Music\\GlueScreenMusic\\wow_main_theme.mp3", 161},
		{L["TITLE_EVILFOREST_NIGHT"], "Sound\\Music\\ZoneMusic\\EvilForest\\NightEvilForest02.mp3", 75},
		{L["TITLE_MOLTENCORE"], "Sound\\Music\\Musical Moments\\battle\\battle01.mp3", 48},
		{L["TITLE_ENCHANTEDFOREST"], "Sound\\Music\\ZoneMusic\\EnchantedForest\\EnchantedForest01.mp3", 50},
		{L["TITLE_DARNASSUS"], "Sound\\Music\\CityMusic\\Darnassus\\Darnassus Walking 1.mp3", 85},
		{L["TITLE_GNOMERAGON"], "Sound\\Music\\CityMusic\\Gnomeragon\\gnomeragon01-zone.mp3", 64},
		{L["TITLE_IRONFORGE"], "Sound\\Music\\CityMusic\\Ironforge\\IronForge Walking 01.mp3", 123},
		{L["TITLE_ORGRIMMAR1"], "Sound\\Music\\CityMusic\\Orgrimmar\\orgrimmar01-zone.mp3", 68},
		{L["TITLE_ORGRIMMAR2"], "Sound\\Music\\CityMusic\\Orgrimmar\\orgrimmar02-zone.mp3", 62},
		{L["TITLE_STORMWIND1"], "Sound\\Music\\CityMusic\\Stormwind\\stormwind07-zone.mp3", 87},
		{L["TITLE_STORMWIND2"], "Sound\\Music\\CityMusic\\Stormwind\\stormwind_highSeas-moment.mp3", 133},
		{L["TITLE_THUNDERBLUFF"], "Sound\\Music\\CityMusic\\Thunderbluff\\Thunderbluff Walking 03.mp3", 121},
		{L["TITLE_UNDERCITY"], "Sound\\Music\\CityMusic\\Undercity\\undercity02-zone.mp3", 85},
		{L["TITLE_BARRENS_DAY"], "Sound\\Music\\ZoneMusic\\BarrenDry\\DayBarrenDry01.mp3", 64},
		{L["TITLE_BARRENS_NIGHT"], "Sound\\Music\\ZoneMusic\\BarrenDry\\NightBarrenDry01.mp3", 67},
		
	},

	-- The Burning Crusade
	i2bc = {
		{L["TITLE_MAIN_THEME"], "Sound\\Music\\GlueScreenMusic\\BC_main_theme.mp3", 226},
		{L["TITLE_SUNWELL_FLYBY"], "Sound\\Music\\ZoneMusic\\Sunwell\\SW_SunwellFlybyUni01.mp3", 57},
		{L["TITLE_ESCDURNHOLDE"], "Sound\\Music\\ZoneMusic\\CavernsOfTime\\CT_EscapeDurnIntroUni.mp3", 112},
		{L["TITLE_HYJAL"], "Sound\\Music\\ZoneMusic\\CavernsOfTime\\CT_HyjalExtWalk8Uni.mp3", 70},
		{L["TITLE_LAMENTOTHIGHBORNE"], "Sound\\Music\\GlueScreenMusic\\BCCredits_Lament_of_the_Highborne.mp3", 171},
		{L["TITLE_SHORELARAN"], "Sound\\Music\\ZoneMusic\\Sunwell\\SW_ShorelAranWalkUni01.mp3", 87},
		{L["TITLE_AZUREMIST"], "Sound\\Music\\ZoneMusic\\Azuremyst\\AV_DraeneiWalkUni01.mp3", 206},
		{L["TITLE_EXODAR"], "Sound\\Music\\ZoneMusic\\Azuremyst\\AI_ExodarWalkUni01.mp3", 109},
		{L["TITLE_THENAGA"], "Sound\\Music\\ZoneMusic\\Azuremyst\\AI_NagaWalkUni03.mp3", 103},
		{L["TITLE_AZUREMIST"], "Sound\\Music\\ZoneMusic\\Azuremyst\\AV_DraeneiWalkUni01.mp3", 206},
		{L["TITLE_BLOODMIST_SATYR"], "Sound\\Music\\ZoneMusic\\Bloodmyst\\BI_SatyrWalkUni01r.mp3", 126},
		{L["TITLE_BLACKTEMPLE"], "Sound\\Music\\ZoneMusic\\BlackTemple\\BT_ArrivalWalkHero01.mp3", 164},
		{L["TITLE_BLACKTEMPLE_GATES"], "Sound\\Music\\ZoneMusic\\BlackTemple\\BT_IllidariWalkHero09.mp3", 74},
		{L["TITLE_BLACKTEMPLE_KARABOR"], "Sound\\Music\\ZoneMusic\\BlackTemple\\BT_KaraborWalkUni04.mp3", 126},
		{L["TITLE_BLACKTEMPLE_RELIQUARY"], "Sound\\Music\\ZoneMusic\\BlackTemple\\BT_ReliquaryWalkHero01.mp3", 112},
		{L["TITLE_BLACKTEMPLE_SANCTUARY"], "Sound\\Music\\ZoneMusic\\BlackTemple\\BT_SanctuaryWalkHero01.mp3", 131},
	},
	
	-- Wrath of the Lich King
	i3wotlk = {
		{L["TITLE_MAIN_THEME"], "Sound\\Music\\GlueScreenMusic\\WotLK_main_title.mp3", 544},
		{L["TITLE_STORMPEAKS"], "Sound\\Music\\ZoneMusic\\StormPeaks\\SP_GeneralWalkB_Day02.mp3", 82},
	},
	
	-- Cataclysm
	i4cata = {
		{L["TITLE_MAIN_THEME"], "Sound\\Music\\cataclysm\\MUS_Shattering_UU01.mp3", 725},
		{L["TITLE_NIGHTELVES"], "Sound\\Music\\cataclysm\\MUS_NightElves_GU01.mp3", 160},
		{L["TITLE_AFTERMATH"], "Sound\\Music\\cataclysm\\MUS_Aftermath_UU01.mp3", 125},
		{L["TITLE_CORRUPTED_DRAGONBLIGHT"], "Sound\\Music\\cataclysm\\MUS_43_CorruptedDragonblight_UU01.mp3", 122},
		{L["TITLE_DRAGONSOUL"], "Sound\\Music\\cataclysm\\MUS_43_DragonSoulA_Moment_04.mp3", 60},
		{L["TITLE_DS_GUNSHIP"], "Sound\\Music\\cataclysm\\MUS_43_Gunships_Hero_UU01.mp3", 189},
		{L["TITLE_FANGS_OT_FATHER"], "Sound\\Music\\cataclysm\\MUS_43_FangsOfTheFatherA_Hero_UU01.mp3", 121},
		{L["TITLE_NIGHTWALK"], "Sound\\Music\\cataclysm\\MUS_43_NightWalk_UU02.mp3", 60},
		{L["TITLE_THRALLBATTLE"], "Sound\\Music\\cataclysm\\MUS_43_ThrallEarthWarder_UU01.mp3", 62},
		{L["TITLE_TWILIGHTSHAMMER"], "Sound\\Music\\cataclysm\\MUS_43_TwilightsHammer_UU01.mp3", 113},
		{L["TITLE_WARSHIPS"], "Sound\\Music\\cataclysm\\MUS_43_Warships_Hero_01.mp3", 90},
		{L["TITLE_WELLOFETERNITY"], "Sound\\Music\\cataclysm\\MUS_43_WellOfEternity_Hero_UU01.mp3", 84},
		{L["TITLE_ZINAZSHARI"], "Sound\\Music\\cataclysm\\MUS_43_ZinAzshari_Hero_UU01.mp3", 177},
		{L["TITLE_RAGE_OT_FIRELANDS"], "Sound\\Music\\cataclysm\\MUS_AdventureA_Hero_UU01.mp3", 141},
		{L["TITLE_ALLIANCE_ONE_HERO"], "Sound\\Music\\cataclysm\\MUS_AllianceHero_UU01.mp3", 195},
	},
	
	-- Mists of Pandaria
	i5mop = {
		{L["TITLE_MAIN_THEME"], "Sound\\Music\\Pandaria\\MUS_50_HeartofPandaria_01.mp3", 480},
		{L["TITLE_WANDERINGISLE"], "Sound\\Music\\ZoneMusic\\WanderingIsle\\MUS_50_WanderingIsleB_Hero_UU01.mp3", 135},
		{L["TITLE_VALLEY4WINDS"], "Sound\\Music\\Pandaria\\MUS_50_BalloonRide_Hero_01.mp3", 165},
		{L["TITLE_SONG_LIULANG"], "Sound\\Music\\Pandaria\\MUS_50_SongofLiulang_VOX_Hero_01.mp3", 170},
		{L["TITLE_SERPENTRACE"], "Sound\\Music\\Pandaria\\MUS_50_SerpentRiders_SerpentRace.mp3", 238},
		{L["TITLE_GINSUVALOR"], "Sound\\Music\\Pandaria\\MUS_50_ShadoPan_A_Hero_03.mp3", 172},
		{L["TITLE_WAYOTMONK"], "Sound\\Music\\Pandaria\\MUS_50_Monk_Windwalker_A_01.mp3", 157},
		
	},
	
	-- Warlords of Draenor
	i6wod = {
		{L["TITLE_MAIN_THEME"], "Sound\\Music\\Draenor\\MUS_60_ASiegeofWorlds_MainTitle.mp3", 726},
		{L["TITLE_A_HEROS_SACRIFICE"], "Sound\\Music\\Draenor\\MUS_60_AHerosSacrifice_v1.mp3", 116},
		{L["TITLE_A_LIGHT_IT_DARKNESS"], "Sound\\Music\\Draenor\\MUS_60_ALightintheDarkness_v1.mp3", 155},
		{L["TITLE_SPIRES_OF_ARAK"], "Sound\\Music\\Draenor\\MUS_60_Arak2_H.mp3", 157},
		{L["TITLE_DRAENOR_ARMY"], "Sound\\Music\\Draenor\\MUS_60_Army_A.mp3", 142},
		{L["TITLE_DRAENOR_FREEDOM"], "Sound\\Music\\Draenor\\MUS_60_BFreedom_H2.mp3", 107},
		{L["TITLE_DRAENOR_BLACKROCK"], "Sound\\Music\\Draenor\\MUS_60_Blackrock_H.mp3", 188},
	},
	
	-- Warcraft III
	i7events = {
		{L["TITLE_POWEROTHORDE"], "Sound\\Music\\ZoneMusic\\DMF_L70ETC01.mp3", 282},
		{L["TITLE_FIREFESTIVAL"], "Sound\\Music\\WorldEvents\\AllianceFirepole.mp3", 68},
		{L["TITLE_ARGENTTOURNY"], "Sound\\Music\\ZoneMusic\\ArgentTournament\\AT_JoustEvent.mp3", 123},
		
	},
}

-- table holding zone info (name, id)
local zonetable = {
	{"Lunarfall", 971},
}

-- Create the options variable to use in the config screen --
local options = {
	name = L["OPT_JUKEBOX_OPTIONS"],
	handler = GarrisonJukeBox,
	type = "group",
	childGroups = "tab",
	args = {
		grp1jukebox = {
			name = L["OPT_JUKEBOX"],
			desc = L["OPT_JUKEBOX_DESC"],
			type = "group",
			order = 1,
			args = {
				hdrselection = {
					name = L["HDR_SELECTION"],
					type = "header",
					order = 1,
				},
				expac = {
					type = "select",
					name = L["SEL_EXPANSION"],
					desc = L["SEL_EXPANSION_DESC"],
					style = "dropdown",
					values = "GetExpanssions",
					set = function(info, value)
						expac = value
					end,
					get = "GetExpac",
					width = "normal",
					order = 2,
				},
				music = {
					type = "select",
					name = L["SEL_MUSIC"],
					desc = L["SEL_MUSIC_DESC"],
					style = "dropdown",
					values = "GetCurExpacMusicList",
					set = function(info, value)
						music = value
					end,
					get = "GetMusic",
					width = "normal",
					order = 5,
				},
				musicadd = {
					type = "execute",
					name = L["CMD_ADD"],
					desc = L["CMD_ADD_DESC"],
					func = function()
						local xps = GarrisonJukeBox:GetExpanssions()
						local xp = xps[GarrisonJukeBox:GetExpac()]
						local mus = GarrisonJukeBox:GetCurExpacMusicList()
						local mu = mus[GarrisonJukeBox:GetMusic()]
						
						-- TODO: Will we allow duplicates?
						table.insert(GarrisonJukeBox.db.profile.musiclist, xp .. "/" .. mu)
						jukebox = xp .. "/" .. mu
					end,
					width = "half",
					order = 10,
				},
				descspace1 = {
					name = " ",
					type = "description",
					width = "full",
					fontSize = "small",
					order = 11,
				},
				cmdplaysamplemusic = {
					type = "execute",
					name = L["OPT_PLAYSAMPLE"],
					desc = L["OPT_PLAYSAMPLE_DESC"],
					func = function()
						local xps = GarrisonJukeBox:GetExpanssions()
						local xp = xps[GarrisonJukeBox:GetExpac()]
						local mus = GarrisonJukeBox:GetCurExpacMusicList()
						local mu = mus[GarrisonJukeBox:GetMusic()]

						StopMusic()
						gMusicPlaying = false
						GarrisonJukeBox:CancelTimer(GarrisonJukeBox.musicTimer)
						PlayMusic(musictable[expac][music][2])
						GarrisonJukeBox:Print(L["NOW_PLAYING"] .. xp .. "/" .. mu)
					end,
					width = "normal",
					order = 15,
				},
				cmdstopsound = {
					type = "execute",
					name = L["CMD_STOP"],
					func = function()
						StopMusic()
						GarrisonJukeBox:ProcessInfo()
					end,
					width = "half",
					order = 16,
				},
				descspace2 = {
					name = " ",
					type = "description",
					width = "full",
					fontSize = "small",
					order = 20,
				},
				hdrjukebox = {
					name = L["SEL_JUKEBOX"],
					type = "header",
					order = 25,
				},
				musiclist = {
					type = "select",
					name = L["SEL_JUKEBOX"],
					desc = L["SEL_JUKEBOX_DESC"],
					style = "dropdown",
					values = function()
						return GarrisonJukeBox.db.profile.musiclist
					end,
					set = function(info, value)
						jukebox = value
					end,
					get = function(info)
						return jukebox
					end,
					width = "double",
					order = 30,
				},
				musicdelete = {
					type = "execute",
					name = L["CMD_REMOVE"],
					desc = L["CMD_REMOVE_DESC"],
					func = function()
						GarrisonJukeBox.db.profile.musiclist[jukebox] = nil
					end,
					width = "half",
					order = 35,
				},
				musicdeleteall = {
					type = "execute",
					name = L["CMD_REMOVEALL"],
					desc = L["CMD_REMOVEALL_DESC"],
					func = function()
						GarrisonJukeBox.db.profile.musiclist = {}
						jukebox = ""
					end,
					width = "normal",
					order = 40,
				},
				descspaceshuffle = {
					name = " ",
					type = "description",
					width = "full",
					fontSize = "medium",
					order = 45,
				},
				shuffle = {
					type = "toggle",
					name = L["TOG_SHUFFLE"],
					desc = L["TOG_SHUFFLE_DESC"],
					set = function(info, value)
						GarrisonJukeBox.db.profile.shuffle = value
					end,
					get = function(info)
						return GarrisonJukeBox.db.profile.shuffle
					end,

					width = "normal",
					order = 50,
				},
				hdrplayer = {
					name = L["HDR_AUDIOPLAYER"],
					type = "header",
					order = 55,
				},
				play = {
					type = "execute",
					name = L["CMD_PLAY"],
					func = function()
						StopMusic()
						GarrisonJukeBox:ProcessInfo()
					end,
					width = "half",
					order = 60,
				},
				prev = {
					type = "execute",
					name = L["CMD_PREV"],
					func = function()
						StopMusic()
						
						gMusicIndex	= gMusicIndex - 1
						if gMusicIndex < 1 then
							gMusicIndex = gMaxMusic
						end
						
						GarrisonJukeBox:ProcessInfo()
					end,
					width = "half",
					order = 65,
				},
				next = {
					type = "execute",
					name = L["CMD_NEXT"],
					func = function()
						StopMusic()
						GarrisonJukeBox:ProcessInfo()
					end,
					width = "half",
					order = 70,
				},
				stop = {
					type = "execute",
					name = L["CMD_STOP"],
					func = function()
						StopMusic()
					end,
					width = "half",
					order = 75,
				},
--[[				hdrselection = {
					name = L["HDR_ZONES"],
					type = "header",
					order = 55,
				},
				i1zone = {
					type = "select",
					name = L["SEL_ZONE"],
					desc = L["SEL_ZONE_DESC"],
					style = "dropdown",
					values = "GetZones",
					set = function(info, value)
						zone = value
					end,
					get = function(info)
						return zone
					end,
					width = "normal",
					order = 60,
				},
				zoneadd = {
					type = "execute",
					name = L["CMD_ADD"],
					desc = L["CMD_ADDZONE_DESC"],
					func = function()
						local zs = GarrisonJukeBox:GetZones()
						local z = zs[zone]
						
						table.insert(GarrisonJukeBox.db.profile.zonelist, z)
					end,
					width = "half",
					order = 65,
				},]]
			},
		},
		grp2audio = {
			name = L["OPT_AUDIO"],
			desc = L["OPT_AUDIO_DESC"],
			type = "group",
			order = 10,
			--disabled = true,
			args = {
				txtplaysound = {
					name = L["INP_WOWAUDIOFILE"],
					type = "input",
					width = "double",
					set = function(info, value)
						playsoundfile = value
					end,

					get = function(info)
						return playsoundfile
					end,
					order = 10,
				},
				cmdplaysound = {
					type = "execute",
					name = L["CMD_PLAY"],
					func = function()
						if doubleslashes == true then
							local s = "" -- temporary string holding newly created sound filename
							local i1, i2 = 1
							local l = strlen(playsoundfile)
							
							-- loop through the filename's characters to find double slashes
							while i1 <= l do
								local i2 = strfind(playsoundfile, "\\\\", i1)
								if i2 ~= nil then
									s = s .. strsub(playsoundfile, i1, i2-1) .. "\\"
									i1 = i2+2
								else
									s = s .. strsub(playsoundfile, i1)
									playsoundfile = s
									break
								end
							end
						end
						
						StopMusic()
						gMusicPlaying = false
						GarrisonJukeBox:CancelTimer(GarrisonJukeBox.musicTimer)
						PlayMusic(playsoundfile)
						GarrisonJukeBox:Print(L["NOW_PLAYING"] .. playsoundfile)
					end,
					width = "half",
					order = 11,
				},
				cmdstopsound = {
					type = "execute",
					name = L["CMD_STOP"],
					func = function()
						StopMusic()
						GarrisonJukeBox:ProcessInfo()
					end,
					width = "half",
					order = 12,
				},
				doubleslashes = {
					type = "toggle",
					name = L["TOG_SLASHES"],
					desc = L["TOG_SLASHES_DESC"],

					set = function(info, value)
						doubleslashes = value
					end,

					get = function(info)
						return doubleslashes
					end,

					width = "full",
					order = 50,
				},
			},
		},
		grp3about = {
			name = L["OPT_ABOUT"],
			desc = "",
			type = "group",
			order = 20,
			args = {
				desctitle = {
					name = setTextColor(L["ADDON_NAME"] .. " " .. gVersion, "SG_GREEN"),
					type = "description",
					width = "full",
					fontSize = "medium",
					order = 1,
				},
				desctitle2 = {
					name = setTextColor(L["ADDON_RELEASE"] .. gReleaseDate, "SG_GREEN"),
					type = "description",
					width = "full",
					fontSize = "medium",
					order = 2,
				},
				descauthor = {
					name = setTextColor(L["ADDON_AUTHOR"] .. " " .. "Azmaedus", "SG_BLUE"),
					type = "description",
					width = "full",
					fontSize = "medium",
					order = 10,
				},
				descspace1 = {
					name = " ",
					type = "description",
					width = "full",
					fontSize = "large",
					order = 20,
				},
				descslashhdr = {
					name = L["ABOUT_SLASHCMD_TITLE"],
					type = "description",
					width = "full",
					fontSize = "medium",
					order = 30,
				},
				descslashplay = {
					name = L["ABOUT_SLASHCMD_PLAY"],
					type = "description",
					width = "full",
					fontSize = "small",
					order = 40,
				},
				descslashprev = {
					name = L["ABOUT_SLASHCMD_PREV"],
					type = "description",
					width = "full",
					fontSize = "small",
					order = 50,
				},
				descslashnext = {
					name = L["ABOUT_SLASHCMD_NEXT"],
					type = "description",
					width = "full",
					fontSize = "small",
					order = 60,
				},
				descslashstop = {
					name = L["ABOUT_SLASHCMD_STOP"],
					type = "description",
					width = "full",
					fontSize = "small",
					order = 70,
				},
--[[				deschowto = {
					name = "Howto",
					type = "description",
					width = "full",
					fontSize = "medium",
					order = 30,
				},
				desccont1 = {
					name = "Jukebox tab",
					type = "description",
					width = "full",
					fontSize = "small",
					order = 40,
				},]]
			},
		},
		next = {
			type = "execute",
			name = "",
			hidden = true,
			func = function()
				StopMusic()
				GarrisonJukeBox:ProcessInfo()
			end,
			width = "half",
			order = 2000,
		},
		stop = {
			type = "execute",
			name = "",
			hidden = true,
			func = function()
				StopMusic()
			end,
			width = "half",
			order = 3000,
		},
		play = {
			type = "execute",
			name = "",
			hidden = true,
			func = function()
				StopMusic()
				GarrisonJukeBox:ProcessInfo()
			end,
			width = "half",
			order = 4000,
		},
		prev = {
			type = "execute",
			name = "",
			hidden = true,
			func = function()
				StopMusic()
				
				gMusicIndex	= gMusicIndex - 1
				if gMusicIndex < 1 then
					gMusicIndex = gMaxMusic
				end
				
				GarrisonJukeBox:ProcessInfo()
			end,
			width = "half",
			order = 5000,
		},
	},
}

-- function to get a table of all expansion/category names with keys that match the options table
function GarrisonJukeBox:GetExpanssions()
	return {
		i1wow = EXPANSION_NAME0,
		i2bc = EXPANSION_NAME1,
		i3wotlk = EXPANSION_NAME2,
		i4cata = EXPANSION_NAME3,
		i5mop = EXPANSION_NAME4,
		i6wod = EXPANSION_NAME5,
		i7events = L["EXP_WCEVENTS"],
	}
end

-- options table functions that may be called from more than one component
function GarrisonJukeBox:GetCurExpacMusicList()
	local tbl = musictable[self:GetExpac()]
	local t = {}

	for k, v in ipairs(tbl) do
		table.insert(t, k, v[1])
	end
	
	return t
end

function GarrisonJukeBox:GetExpac(info)
	return expac
end

function GarrisonJukeBox:GetMusic(info)
	return music
end

function GarrisonJukeBox:GetZones()
	local tbl = {}
	for i = 1, #zonetable do
		table.insert(tbl, zonetable[i][1])
	end
	return tbl
end

--[[=============================================================
						OnInitialize
    =============================================================]]
function GarrisonJukeBox:OnInitialize()
	GarrisonJukeBox:RegisterEvent("PLAYER_ENTERING_WORLD", "OnPEWEvent")
	GarrisonJukeBox:RegisterEvent("ZONE_CHANGED_NEW_AREA", "OnZoneEvent")
	GarrisonJukeBox:RegisterEvent("ZONE_CHANGED_INDOORS", "OnZoneEvent")
	GarrisonJukeBox:RegisterEvent("ZONE_CHANGED", "OnZoneEvent")
		
	GarrisonJukeBox.db = LibStub("AceDB-3.0"):New("GarrisonJukeBoxDB", defaults, true)
	LibStub("AceConfig-3.0"):RegisterOptionsTable("GarrisonJukeBoxDB", options, "gjb")
	GarrisonJukeBox.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("GarrisonJukeBoxDB", "Garrison JukeBox")
	self:RegisterChatCommand("gjb", "ChatCommand")

	gMusicIndex = 1
	gMusicPlaying = false
	gMaxMusic = 0
	self.musicTimer = nil

	self:Print(L["ADDON_NAME"] .. " " .. gVersion .. " " .. L["ADDON_LOADED"])
end

-- -------------------------------------------
-- Events
-- -------------------------------------------
function GarrisonJukeBox:OnPEWEvent(event, ...)
	StopMusic()
	gMusicPlaying = false
	self.musicTimer = self:ScheduleTimer("TimerEvent", gOnLoadMusicTimer)
end

function GarrisonJukeBox:OnZoneEvent(event, ...)
	self:ProcessInfo()
end

-- -------------------------------------------
-- Ace3 Timer event for playing music
-- -------------------------------------------
function GarrisonJukeBox:TimerEvent()
	self:ProcessInfo()
end

-- -------------------------------------------
-- slashcmd
-- -------------------------------------------
function GarrisonJukeBox:ChatCommand(input)
	-- Open dialog if no parameters, else do something with the parameters
	if not input or input:trim() == "" then
		LibStub("AceConfigDialog-3.0"):Open("GarrisonJukeBoxDB")
		--InterfaceOptionsFrame_OpenToCategory(GarrisonJukeBox.optionsFrame)
		-- InterfaceOptionsFrame_OpenToCategory seems to not function immediately. Firts attempt == Blizz options frame. Second attempt == gjb options frame ???
	else
		LibStub("AceConfigCmd-3.0").HandleCommand(GarrisonJukeBox, "gjb", "GarrisonJukeBoxDB", input)
	end
end

-- -------------------------------------------
-- function called to determine if music needs to be played after a registered event has fired
-- -------------------------------------------
function GarrisonJukeBox:ProcessInfo()
	if GetCurrentMapAreaID() == LUNARFALL then
		gMaxMusic = table.getn(GarrisonJukeBox.db.profile.musiclist)
		if gMaxMusic > 0 then
			self:PlayMusicList()
		end
	else 
		if gMusicPlaying == true then
			StopMusic()
			gMusicPlaying = false
			self:CancelTimer(self.musicTimer)
		end
	end
end

-- -------------------------------------------
-- Table Invert for query of key
-- -------------------------------------------
function table_invert(t)
   local s={}
   for k,v in pairs(t) do
     s[v]=k
   end
   return s
end

-- -------------------------------------------
-- break appart the musiclist item to find out what file to play in the musicfile table
-- returns a table representing the music info
-- -------------------------------------------
function MusicInfoFromJukeBoxItem(s)
	local idx = 0
	
	-- get the expansion key
	local b = strfind(s, "/")
	local ns = strsub(s, 1, b-1)
	local xp = GarrisonJukeBox:GetExpanssions() -- 
	local xpi = table_invert(xp)
	idx = xpi[ns]

	-- get music info
	ns = strsub(s, b+1)
	local mt = musictable[idx] -- get table of music infos relative to the expansion
	local t = {}
	
	-- loop through tables to find the music info
	for k, v in ipairs(mt) do
		if v[1] == ns then
			t[1] = v[1]
			t[2] = v[2]
			t[3] = v[3]
			break
		end
	end
	
	return t
end

-- -------------------------------------------
-- function to play an item in the musiclist table
-- -------------------------------------------
function GarrisonJukeBox:PlayMusicList()
	gMaxMusic = table.getn(GarrisonJukeBox.db.profile.musiclist)
	if gMaxMusic > 0 then
		self:CancelTimer(self.musicTimer)
		
		-- check if the list has shrunk since last pass and index is greater than number of list items
		if gMusicIndex > gMaxMusic then
			gMusicIndex = gMaxMusic
		end
		
		-- check if the playlist is in shuffle mode
		if GarrisonJukeBox.db.profile.shuffle == true then
			gMusicIndex = math.random(1, gMaxMusic)
		end
		
		-- find what music to play
		local mli = GarrisonJukeBox.db.profile.musiclist[gMusicIndex]
		local info = MusicInfoFromJukeBoxItem(mli) -- gets a table from musictable[n]
		StopMusic()
		PlayMusic(info[2])
		self:Print(L["PLAYING"] .. "#" .. gMusicIndex .. " - " .. GarrisonJukeBox.db.profile.musiclist[gMusicIndex])
		gMusicPlaying = true
		self.musicTimer = self:ScheduleTimer("TimerEvent", info[3])

		-- increment index after item is playing
		if gMusicIndex == gMaxMusic then
			gMusicIndex = 1
		else
			gMusicIndex = gMusicIndex + 1
		end
	end
end
